@@ -1,6 +1,12 @@
 /*****************************************************************************
- * Author:  Fredrik Hbinette <hubbe@hubbe.net>
- *          Peter Leese <peter@leese.net>
+ *
+ * Original author: Fredrik Hbinette <hubbe@hubbe.net>
+ *
+ * Current Authors: Louis Bavoil <bavoil@cs.utah.edu>
+ *                  Peter Leese <hubbe@hubbe.net>
+ *
+ * This code is based on and is a branch of plugger written 
+ * by Fredrik Hbinette <hubbe@hubbe.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,8 +21,10 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ *
  *****************************************************************************/
 
+#include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
 #include <stdio.h>
@@ -63,6 +71,10 @@ static int repeatsResetVal;
 
 static int pipe_fd;
 static int flags;
+static int sig_fds[2];
+
+#define SIG_RD_FD 0
+#define SIG_WR_FD 1
 
 static GC gc_white;
 static GC gc_black;
@@ -178,9 +190,9 @@ static void redraw(void)
 
      if(old_buttonsize != buttonsize)
      {
-	  old_buttonsize=buttonsize;
-	  XFillRectangle(dpy, topLevel, gc_white,
-			 0,0,(unsigned)attr.width, (unsigned)attr.height);
+          old_buttonsize=buttonsize;
+          XFillRectangle(dpy, topLevel, gc_white,
+                         0,0,(unsigned)attr.width, (unsigned)attr.height);
      }
     
 
@@ -211,7 +223,7 @@ static void redraw(void)
           int i;
 
           XFillPolygon(dpy, topLevel, c, points, 6,
-		  Convex, CoordModeOrigin);
+                Convex, CoordModeOrigin);
 
           points[2].y++;
           points[5].x++;
@@ -233,7 +245,7 @@ static void redraw(void)
      else
      {   
           XFillPolygon(dpy, topLevel, c, points, 6,
-		  Convex, CoordModeOrigin);
+                 Convex, CoordModeOrigin);
 
           x = points[0].x;
           y = points[0].y;
@@ -304,56 +316,65 @@ static void my_play(char * command)
 {
      if(state != STATE_STOP)
      {
-	  if(!kill(-pid, SIGCONT))
-	  {
-	       state=STATE_PLAY;
-	       return;
-	  }
+          if(!kill(-pid, SIGCONT))
+          {
+               state=STATE_PLAY;
+               return;
+          }
      }
 
      pid=fork();
      if(pid == -1)
      {
-	  state=STATE_STOP;
-	  return;
+          state=STATE_STOP;
+          return;
      }
      if(pid == 0)
      {
-	  char *cmd[4];
+          char *cmd[4];
           /* Child process, so as usually need to cleanup any inherited
            * file descriptors */
           if(dpy)
           {
                close(ConnectionNumber(dpy));
           }
+          if(sig_fds[SIG_RD_FD] >= 0)
+          {
+              close(sig_fds[SIG_RD_FD]);
+          }
+          if(sig_fds[SIG_WR_FD] >= 0)
+          {
+              close(sig_fds[SIG_WR_FD]);
+          }
+          close(pipe_fd);
 
           /* Redirect stdout & stderr to /dev/null */
           if(flags & H_NOISY)
           {
-	        const int ofd = open("/dev/null", O_RDONLY);
-	        if(ofd)
+                const int ofd = open("/dev/null", O_RDONLY);
+                if(ofd)
                 {
-	             dup2(ofd, 1);
-	             dup2(ofd, 2);
-	             close(ofd);
+                     dup2(ofd, 1);
+                     dup2(ofd, 2);
+                     close(ofd);
                 }
           }
 
-	  setpgid(pid, 0);
+          setpgid(pid, 0);
 
-	  cmd[0]="/bin/sh";
-	  cmd[1]="-c";
-	  cmd[2]=command;
-	  cmd[3]=0;
-	  execvp(cmd[0], cmd);
-	  exit(EX_UNAVAILABLE);
+          cmd[0]="/bin/sh";
+          cmd[1]="-c";
+          cmd[2]=command;
+          cmd[3]=0;
+          execvp(cmd[0], cmd);
+          exit(EX_UNAVAILABLE);
      }
      else
      {
-	  state=STATE_PLAY;
-	  if(!repeats) 
+          state=STATE_PLAY;
+          if(!repeats) 
               repeats = repeatsResetVal;
-	  if((repeats > 0) && (repeats != MAXINT)) 
+          if((repeats > 0) && (repeats != MAXINT)) 
               repeats--;
      }
 }
@@ -371,12 +392,12 @@ static void my_pause(char * command)
 {
      if(state != STATE_STOP)
      {
-	  if(!kill(-pid, SIGSTOP))
-	       state = STATE_PAUSE;
-	  else
-	       state = STATE_STOP;
+          if(!kill(-pid, SIGSTOP))
+               state = STATE_PAUSE;
+          else
+               state = STATE_STOP;
 
-	  return;
+          return;
      }
 }
 
@@ -407,9 +428,9 @@ static void my_stop(char * command)
      if(state == STATE_PAUSE) my_play(command);
      if(state == STATE_PLAY)
      {
-	  if(pid > 0) my_kill(-pid);
-	  state=STATE_STOP;
-	  repeats=0;
+          if(pid > 0) my_kill(-pid);
+          state=STATE_STOP;
+          repeats=0;
      }
 }
 
@@ -467,22 +488,96 @@ static void sigdie(int sig)
 
 /******************************************************************************/
 /**
- * @brief Callback filter for X events
+ * @brief Convert signal to message in pipe
  *
- * Defines the X events that we wish to see. Returns true for everything. 
+ * Callback function attached as a signal handler. This sends a message over
+ * a pipe about the signal that has just happened. This allows signals to
+ * be handled by select() 
  *
- * @param[in] dpy The display pointer
- * @param[in] ev  Pointer to the event data
- * @param[in] arg Any associated event data
- *
- * @return Always returns true
+ * @param[in] sig The signal
  *
  *****************************************************************************/
-static Bool AllXEventsPredicate(Display *dpy, XEvent *ev, char *arg)
+static void sig_to_fd(int sig)
 {
-     return True;
+    D("Signal %i routed to pipe\n", sig);
+    if(write(sig_fds[SIG_WR_FD], &sig, sizeof(int)) != sizeof(int))
+    {
+        D("Signal write to pipe failed\n");
+    }
 }
 
+/******************************************************************************/
+/**
+ * Checks for X events and processes them. Returns when no more events left
+ * to process.
+ *
+ * @return nothing
+ *
+ *****************************************************************************/
+static void check_x_events(char * command)
+{
+     int numEvents = XPending(dpy);
+
+     while(numEvents > 0)
+     {
+          XEvent ev;
+
+          XNextEvent(dpy, &ev);
+
+          switch(ev.type)
+          {
+          case ButtonPress:
+               if(ev.xbutton.button == 1)
+               {
+                    buttonDown = (ev.xbutton.x - bx) / buttonsize;
+                    redraw();  /* do first gives quicker visual feedback */
+                    XSync(dpy, False);
+
+                    switch(buttonDown)
+                    {
+                    case 0: /* play */ 
+                         my_play(command); 
+                         break;
+                    case 1: /* pause*/ 
+                         my_pause(command);
+                         break;
+                    case 2: /* stop */ 
+                         my_stop(command); 
+                         break;
+                    }
+               }
+               break;
+
+          case ButtonRelease:
+               if(buttonDown != -1)
+               {   
+                    buttonDown = -1;
+                    redraw();
+               }     
+               break;
+
+          case Expose:
+               if(ev.xexpose.count)
+                    break;
+          case ResizeRequest:
+          case MapNotify:
+               redraw();
+               break;
+
+          default:
+               D("Unknown event %d\n",ev.type);
+               break;
+          }
+
+          /* If this is the last of this batch, check that more havent
+           * been added in the meantime as a result of an action */
+          numEvents--;
+          if(numEvents == 0)
+          {
+               numEvents = XPending(dpy);
+          }
+     }
+}
 
 /******************************************************************************/
 /** 
@@ -508,7 +603,7 @@ static void normalise_window_coords(unsigned * width, unsigned * height,
     if(smallControls)
     {
         *y = *x = 0;
-	*width  = MIN_CONTROLS_WIDTH;
+        *width  = MIN_CONTROLS_WIDTH;
         *height = MIN_CONTROLS_HEIGHT;
         return;
     }
@@ -557,19 +652,19 @@ static void check_pipe_fd_events(void)
      n = read(pipe_fd, ((char *)& wintmp),  sizeof(wintmp));
      if (n < 0)
      {
-	  if (errno == EINTR)
+          if (errno == EINTR)
                return;
-	  exit(EX_UNAVAILABLE);
+          exit(EX_UNAVAILABLE);
      }
   
      if (n == 0)
      {
-	  exit(EX_UNAVAILABLE);
+          exit(EX_UNAVAILABLE);
      }
   
      if (n != sizeof(wintmp))
      {
-	  return;
+          return;
      }
 
      /* Adjust the width & height to compensate for the window border width */
@@ -647,14 +742,14 @@ int main(int argc, char **argv)
      }
 
      i = sscanf(argv[1],"%d,%d,%d,%lu,%d,%d,%d,%d",
-	    &flags,
-	    &repeatsResetVal,
-	    &pipe_fd,
-	    &temp,
-	    (int *)&x,  /* Not needed and not used */
-	    (int *)&y,  /* Not needed and not used */
-	    (int *)&width,
-	    (int *)&height);
+            &flags,
+            &repeatsResetVal,
+            &pipe_fd,
+            &temp,
+            (int *)&x,  /* Not needed and not used */
+            (int *)&y,  /* Not needed and not used */
+            (int *)&width,
+            (int *)&height);
 
      if(i < 8)
      {
@@ -673,14 +768,14 @@ int main(int argc, char **argv)
      /* The application will use the $repeat variable */
      if (flags & H_REPEATCOUNT)
      {   
-	  repeatsResetVal = 1;
+          repeatsResetVal = 1;
      }
 
      if(!(dpy = XOpenDisplay(getenv("DISPLAY"))))
      {
-	  D("%s: unable to open display %s\n",
-	         argv[0], XDisplayName(getenv("DISPLAY")));
-	  exitEarly();
+          D("%s: unable to open display %s\n",
+                   argv[0], XDisplayName(getenv("DISPLAY")));
+          exitEarly();
      }
 
      /* If the command contains a reference to window it is likely that that
@@ -688,7 +783,7 @@ int main(int argc, char **argv)
       * so lets use small controls in top left corner */
      if(strstr(command,"$window") || strstr(command,"$hexwindow"))
      {
-	  smallControls=1;
+          smallControls=1;
      }
 
      /* Adjust the width & height to compensate for the window border
@@ -716,17 +811,17 @@ int main(int argc, char **argv)
      attr.override_redirect=0;
 
      topLevel = XCreateWindow(dpy,
-			      parentWid,
-			      x, y,
-			      width, height, 
+                              parentWid,
+                              x, y,
+                              width, height, 
                               WINDOW_BORDER_WIDTH, 
                               CopyFromParent,
-			      InputOutput, (Visual *) CopyFromParent,
-			      (unsigned long)(CWBorderPixel|
-			       CWEventMask|
-			       CWOverrideRedirect|
-			       CWBackPixel),
-			      &attr);
+                              InputOutput, (Visual *) CopyFromParent,
+                              (unsigned long)(CWBorderPixel|
+                              CWEventMask|
+                              CWOverrideRedirect|
+                              CWBackPixel),
+                              &attr);
 
      classhint.res_name = "mozplugger-controller";
      classhint.res_class = "mozplugger-controller";
@@ -770,100 +865,79 @@ int main(int argc, char **argv)
 
      old_state=state;
 
-  
+     if(pipe(sig_fds) != 0)
+     {
+          D("Failed to create pipe to handle signals");
+          sig_fds[SIG_RD_FD] = sig_fds[SIG_WR_FD] = -1;
+     }
+
      signal(SIGHUP, sigdie);
      signal(SIGINT, sigdie);
      signal(SIGTERM, sigdie);
 
+     if(sig_fds[SIG_WR_FD] >= 0)
+     {
+          /* Re-direct SIGCHLD events through a pipe to be see by select() */
+          signal(SIGCHLD, sig_to_fd);
+     }
 
      if(igetenv("autostart",1))
-	  my_play(command);
+          my_play(command);
 
      while(1)
      {
-	  struct timeval tv;
-	  fd_set fds;
-	  XEvent ev;
+          fd_set fds;
           int maxFd;
 
-	  if(state != old_state)
-	  {
-	       redraw();
-	       old_state=state;
-	  }
-
-	  FD_ZERO(&fds);
-	  FD_SET(ConnectionNumber(dpy),&fds);
-	  FD_SET(pipe_fd,&fds);
-
-          maxFd = ConnectionNumber(dpy);
-          maxFd = MAX(maxFd, pipe_fd);
-
-	  tv.tv_sec = 0;
-	  tv.tv_usec = 1000000/5; /* Check if the subprocess died 5 times / sec */
-
-	  if( select(maxFd + 1, &fds, NULL, NULL, &tv) > 0)
+          if(state != old_state)
           {
-	       if (FD_ISSET(pipe_fd, &fds))
-               {    
-	            check_pipe_fd_events();
-               }
+               redraw();
+               old_state=state;
           }
 
-	  if(pid != -1)
-	  {
-	       int status;
-	       if(waitpid(pid, &status, WNOHANG) > 0)
-	       {
-		    pid=-1;
-		    if(state == STATE_PLAY)
-		    {
-			 state = STATE_STOP;
-			 if(repeats > 0) my_play(command);
-		    }
-	       }
-	  }
+          check_x_events(command);
 
-	  while(XCheckIfEvent(dpy, &ev, AllXEventsPredicate, NULL))
-	  {
-	       switch(ev.type)
-	       {
-	       case ButtonPress:
-		    if(ev.xbutton.button == 1)
-		    {
-			 buttonDown = (ev.xbutton.x - bx) / buttonsize;
-                         redraw();  /* do first gives quicker visual feedback */
-                         XSync(dpy, False);
+          FD_ZERO(&fds);
 
-                         switch(buttonDown)
-			 {
-			 case 0: /* play */ my_play(command); break;
-			 case 1: /* pause*/ my_pause(command);break;
-			 case 2: /* stop */ my_stop(command); break;
-			 }
-		    }
-		    break;
-	 
-               case ButtonRelease:
-                    if(buttonDown != -1)
-                    {   
-                        buttonDown = -1;
-                        redraw();
-                    }     
-                    break;
+          FD_SET(ConnectionNumber(dpy), &fds);
+          FD_SET(pipe_fd, &fds);
 
-	       case Expose:
-		    if(ev.xexpose.count) break;
-	       case ResizeRequest:
-	       case MapNotify:
-		    redraw();
-		    break;
-	  
-	       default:
-		    D("Unknown event %d\n",ev.type);
-                    break;
-	       }
-	  }
+          maxFd = MAX(ConnectionNumber(dpy), pipe_fd);
+
+          if(sig_fds[SIG_RD_FD] >= 0)
+          {
+               FD_SET(sig_fds[SIG_RD_FD], &fds);
+               maxFd = MAX(maxFd, sig_fds[SIG_RD_FD]);
+          }
+
+          D("SELECT IN maxFd = %i\n", maxFd);
+          if( select(maxFd + 1, &fds, NULL, NULL, 0) > 0)
+          {
+               if (FD_ISSET(pipe_fd, &fds))
+               {    
+                    check_pipe_fd_events();
+               }
+               if(FD_ISSET(sig_fds[SIG_RD_FD], &fds))
+               {
+                   int sig;
+                   read(sig_fds[SIG_RD_FD], &sig, sizeof(int));
+               }
+          }
+          D("SELECT OUT\n");
+
+          if(pid != -1)
+          {
+               int status;
+               if(waitpid(pid, &status, WNOHANG) > 0)
+               {
+                    pid=-1;
+                    if(state == STATE_PLAY)
+                    {
+                         state = STATE_STOP;
+                         if(repeats > 0) my_play(command);
+                    }
+               }
+          }
      }
 }
 
