@@ -1,7 +1,12 @@
 /*****************************************************************************
- * Authors: Louis Bavoil <bavoil@cs.utah.edu>
- *          Peter Leese <peter@leese.net>
- *          Fredrik Hbinette <hubbe@hubbe.net>
+ *
+ * Original Author:  Fredrik Hbinette <hubbe@hubbe.net>
+ *
+ * Current Authors: Louis Bavoil <bavoil@cs.utah.edu>
+ *                  Peter Leese <peter@leese.net>
+ *
+ * This code is based on and is a branch of plugger written 
+ * by Fredrik Hbinette <hubbe@hubbe.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,10 +21,12 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ *
  *****************************************************************************/
 
 #define _GNU_SOURCE /* for getsid() */
 
+#include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
 #include <stdio.h>
@@ -43,6 +50,7 @@ static struct
 {
      int    mapped;
      int    reparented;
+     int    reparentedAttemptCount;
      Window window;
      int    borderWidth;
      int    x;
@@ -69,11 +77,18 @@ static int repeats;
 static char *winname;
 static char *command;
 static char *file;
+static int sig_fds[2];
+
+#define SIG_RD_FD  0
+#define SIG_WR_FD  1
 
 static XWindowAttributes wattr;
 
+#ifdef USE_MUTEX_LOCK
 static Atom swallowMutex;
 static int  swallowMutexTaken;
+#endif
+
 static Atom windowOwnerMark;
 
 static int xaspect;
@@ -107,6 +122,7 @@ static int error_handler(Display *dpy, XErrorEvent *err)
      return 0;
 }
 
+#ifdef USE_MUTEX_LOCK
 /*****************************************************************************/
 /**
  * @brief Initialise semaphore
@@ -125,7 +141,18 @@ static void initSwallowMutex(void)
 
     swallowMutex = XInternAtom(display, "MOZPLUGGER_SWALLOW_MUTEX", 0);
     swallowMutexTaken = 0;
+}
+#endif
 
+/*****************************************************************************/
+/**
+ * Initialise the X atom used to mark windows as owned by mozplugger
+ *
+ * @return None
+ *
+ *****************************************************************************/
+static void initWindowOwnerMarker(void)
+{
     windowOwnerMark = XInternAtom(display, "MOZPLUGGER_OWNER", 0);
 }
 
@@ -214,6 +241,7 @@ static int getOwnerFromProperty(Window w, Atom name, uint32 * hostId, uint32 * p
     return success;
 }
 
+#ifdef USE_MUTEX_LOCK
 /*****************************************************************************/
 /**
  * @brief Get owner of mutex semaphore
@@ -384,6 +412,7 @@ static void giveSwallowMutex()
     XDeleteProperty(display, wattr.root, swallowMutex);
     swallowMutexTaken = 0;
 }
+#endif
 
 /*****************************************************************************/
 /**
@@ -603,7 +632,12 @@ static void change_leader(void)
 /**
  * @brief Reparent window
  *
- * Reparent the window
+ * Reparent the window, a count is kept of the number of attempts. If this
+ * exceeds 10, give up (i.e. avoid two instances of helper fighting over
+ * the same application). Originally this was done with a semaphore, but this
+ * caused more problems than it fixed. So alternative is to assume that it is
+ * such an unlikely occurance that two instances of helper will run at exactly
+ * the same time and try to get the same window, that we give up.
  *
  * @return 1(true) if success, 0(false) if not
  *
@@ -613,6 +647,14 @@ static int reparent_window(void)
      if (!victimDetails.window) return 0;
      if (!parentDetails.window) return 0;
 
+     if(victimDetails.reparentedAttemptCount > 10)
+     {
+          D("Giving up reparenting to avoid - tried 10 times\n");
+          return 0;
+     }
+
+     victimDetails.reparentedAttemptCount++;
+
      XSelectInput(display, victimDetails.window, StructureNotifyMask); 
      XSync(display, False);
 
@@ -704,18 +746,19 @@ static int my_strcmp(char *windowname, char *name)
  *****************************************************************************/
 static char check_window_name(Window w, char *name)
 {
-     char *windowname;
+     char * windowname;
      XClassHint windowclass;
-     char match;
 
      D("XFetchName\n");
      if (XFetchName(display, w, &windowname))
      {
-	  D("Winrecur, checking window NAME %x (%s != %s)\n",
-	    (unsigned)w, windowname, name);
-	  match = (my_strcmp(windowname, name) == 0);
+	  const char match = (my_strcmp(windowname, name) == 0);
+
+          D("Winrecur, checking window NAME %x (%s %s %s)\n",
+                            (unsigned)w, windowname, match ? "==" : "!=", name);
 	  XFree(windowname);
-	  if (match) {
+	  if (match) 
+          {
 	       return 1;
 	  }
      }
@@ -723,11 +766,13 @@ static char check_window_name(Window w, char *name)
      D("XGetClassHint\n");
      if (XGetClassHint(display, w, &windowclass))
      {
-	  D("Winrecur, checking window CLASS %x (%s != %s)\n",
-	    (unsigned)w, windowclass.res_name, name);
-	  match = (my_strcmp(windowclass.res_name, name) == 0);
+	  const char match = (my_strcmp(windowclass.res_name, name) == 0);
+
+          D("Winrecur, checking window CLASS %x (%s %s %s)\n",
+                  (unsigned)w, windowclass.res_name, match ? "==" : "!=", name);
 	  XFree(windowclass.res_class);
 	  XFree(windowclass.res_name);
+
           return match;
      }
      return 0;
@@ -818,7 +863,8 @@ static Window find_winrecur(Window from, int depth, char *name)
 	       if (ignored[e] == from)
                {
                     /* This window is OLD, ignore it (and its children) */
-	            return (Window)0;
+                    D("find_winrecur: Ignore %x, old window\n", (unsigned)from);
+	            return 0;
                }
           }
 
@@ -826,7 +872,7 @@ static Window find_winrecur(Window from, int depth, char *name)
            * not mark it as ours */
           if(chkAndMarkVictimWindow(from))
           {
-	        return from;
+               return from;
           }
      }
 
@@ -941,6 +987,14 @@ static void run_app(char **argv)
      {
           close(ConnectionNumber(display));
      }
+     if(sig_fds[SIG_RD_FD] >= 0)
+     {
+          close(sig_fds[SIG_RD_FD]);
+     }
+     if(sig_fds[SIG_WR_FD] >= 0)
+     {
+          close(sig_fds[SIG_WR_FD]);
+     }
      close(pipe_fd);
 
      /* Redirect stdout & stderr to /dev/null */
@@ -1015,8 +1069,9 @@ static void find_victim(Window window)
 
 	       /* We don't need to listen for new windows anymore */
 	       XSelectInput(display, wattr.root, 0);
+#ifdef USE_MUTEX_LOCK
 	       giveSwallowMutex();
- 	
+#endif 	
 	       if (flags & H_MAXASPECT)
 	       {
 		    set_aspect(ca.width, ca.height);
@@ -1026,37 +1081,26 @@ static void find_victim(Window window)
      }
 }
 
-/******************************************************************************/
-/**
- * @brief Callback for filtering X events
- *
- * Called from X to filter events, in this case always returns true
- *
- * @param[in] dpy Pointer to display
- * @param[in] ev  Pointer to event
- * @param[in] arg Pointer to any event data
- *
- * @return Always returns true
- *
- *****************************************************************************/
-static Bool AllXEventsPredicate(Display *dpy, XEvent *ev, char *arg)
-{
-     return True;
-}
-
 /******************************************************************************/
 /**
  * @brief Check events from X
  *
- * Read in events from X and process
+ * Read in events from X and process, keep reading in events until no more
+ * and then exit. It is important that all events have been processed and
+ * not left pending.
  *
  *****************************************************************************/
 static void check_x_events(void)
 {
-     XEvent ev;
+     int numEvents = XPending(display);
 
-     while (XCheckIfEvent(display, &ev, AllXEventsPredicate, NULL))
+     /* While some events pending... Get and action */
+     while (numEvents > 0)
      {
+          XEvent ev;
+
+          XNextEvent(display, &ev);
+         
 	  if (ev.xany.window == wattr.root)
 	  {
 	       switch (ev.type)
@@ -1072,7 +1116,8 @@ static void check_x_events(void)
 		    break;
 
 	       case ConfigureNotify:
-		    D("***ConfigureNotify for root: %x\n", (unsigned) ev.xconfigure.window);
+		    D("***ConfigureNotify for root: %x\n", 
+                                               (unsigned) ev.xconfigure.window);
 		    find_victim(ev.xconfigure.window);
 		    break;
 
@@ -1104,7 +1149,9 @@ static void check_x_events(void)
 			 D("REPARENT NOTIFY on victim to the right window\n");
 			 adjust_window_size();
 			 victimDetails.reparented = 1;
-		    } else {
+		    } 
+                    else 
+                    {
 			 D("REPARENT NOTIFY on victim to some other window!\n");
 			 reparent_window();
 			 victimDetails.reparented = 0;
@@ -1172,15 +1219,22 @@ static void check_x_events(void)
 	  }
           else
           {
-	       D("!!Got unhandled event for uknown->%d\n", ev.type);
+	       D("!!Got unhandled event for unknown->%d\n", ev.type);
           }
-     }
 
-     /* For Window Maker */
-     if (victimDetails.reparented && !victimDetails.mapped) 
-     {
-	  D("XMapWindow\n");
-	  XMapWindow(display, victimDetails.window);
+          /* If this is the last of this batch, check that more havent
+           * been added in the meantime as a result of an action */
+          numEvents--;
+          if(numEvents == 0)
+          {
+               /* For Window Maker */
+               if (victimDetails.reparented && !victimDetails.mapped) 
+               {
+	            D("XMapWindow\n");
+	            XMapWindow(display, victimDetails.window);
+               }
+               numEvents = XPending(display);
+          }
      }
 }
 
@@ -1205,14 +1259,18 @@ static void check_pipe_fd_events(void)
      {
 	  if (errno == EINTR) return;
 	  D("Winddata read error, exiting\n");
+#ifdef USE_MUTEX_LOCK
           giveSwallowMutex();
+#endif
 	  exit(EX_UNAVAILABLE);
      }
   
      if (n == 0)
      {
 	  D("Winddata EOF, exiting\n");
+#ifdef USE_MUTEX_LOCK
           giveSwallowMutex();
+#endif
 	  exit(EX_UNAVAILABLE);
      }
   
@@ -1240,43 +1298,90 @@ static void check_pipe_fd_events(void)
 
 /******************************************************************************/
 /**
- * @brief Check all events
+ * Keep checking all events until application dies
  *
  * Use select to check if any events need processing
  *
  *****************************************************************************/
-static void check_all_events(void)
+static void check_all_events(pid_t pid)
 {
-     struct timeval tv;
      int maxfd;
      fd_set fds;
+     int status;
 
-     FD_ZERO(&fds);
-     FD_SET(ConnectionNumber(display), &fds);
-     FD_SET(pipe_fd, &fds);
-
-     maxfd = MAX(ConnectionNumber(display), pipe_fd);
-     
-     tv.tv_sec = SELECT_TIMEOUT_SEC;
-     tv.tv_usec = SELECT_TIMEOUT_USEC;
-     
-#if 0
-     D("SELECT IN\n");
-#endif
-
-     if (select(maxfd + 1, &fds, NULL, NULL, &tv) > 0)
+     while(1)
      {
-	  D("SELECT OUT\n");
-	  if (FD_ISSET(pipe_fd, &fds))
-	       check_pipe_fd_events();
-     }
+          if (display)
+          {
+	       check_x_events();
+          }
 
-     if (display)
-     {
-	  check_x_events();
+          FD_ZERO(&fds);
+          FD_SET(ConnectionNumber(display), &fds);
+          FD_SET(pipe_fd, &fds);
+
+          maxfd = MAX(ConnectionNumber(display), pipe_fd);
+
+          if(sig_fds[SIG_RD_FD] >= 0)
+          {
+              FD_SET(sig_fds[SIG_RD_FD], &fds);
+              maxfd = MAX(maxfd, sig_fds[SIG_RD_FD]);
+          }
+     
+          D("SELECT IN\n");
+          if (select(maxfd + 1, &fds, NULL, NULL, 0) > 0)
+          {
+	      if (FD_ISSET(pipe_fd, &fds))
+              {
+	           check_pipe_fd_events();
+              }
+              if( FD_ISSET(sig_fds[SIG_RD_FD], &fds))
+              {
+                  int sig;
+                  read(sig_fds[SIG_RD_FD], &sig, sizeof(sig));
+              }  
+          }
+          else
+          {
+              D("Select exited unexpectly, errno = %i\n", errno);
+          }
+          D("SELECT OUT\n");
+
+          if(pid >= 0)
+          {
+	       if(waitpid(pid, &status, WNOHANG))
+               {
+                    pid = -1;     
+                    if((flags & H_DAEMON) == 0)
+                    {
+                         return;
+                    }
+                    D("check_all_events: child process has died, keep going\n");
+               }
+          }
      }
 }
 
+/******************************************************************************/
+/**
+ * @brief Convert signal to message in pipe
+ *
+ * Callback function attached as a signal handler. This sends a message over
+ * a pipe about the signal that has just happened. This allows signals to
+ * be handled by select() 
+ *
+ * @param[in] sig The signal
+ *
+ *****************************************************************************/
+static void sig_to_fd(int sig)
+{
+    D("Signal %i routed to pipe\n", sig);
+    if(write(sig_fds[SIG_WR_FD], &sig, sizeof(int)) != sizeof(int))
+    {
+        D("Signal write to pipe failed\n");
+    }
+}
+
 /******************************************************************************/
 /**
  * @brief handle the application
@@ -1293,12 +1398,12 @@ static void handle_app(pid_t pid)
      if (flags & H_SWALLOW)
      {
           /* Whilst waiting for the Application to complete, check X events */
-	  while (!waitpid(pid, &status, WNOHANG))
-	  {
-	       check_all_events();
-	  }
+	  check_all_events(pid);
+           
+#ifdef USE_MUTEX_LOCK
           /* Make sure the semaphore has been released */
           giveSwallowMutex();
+#endif
      }
      else if(flags & H_DAEMON)
      {
@@ -1393,6 +1498,20 @@ int main(int argc, char **argv)
        file,
        command);
 
+     /* Create pipe to handle the signal handler to select interface */
+     if(pipe(sig_fds) != 0)
+     {
+          D("Failed to create pipe to handle signals");
+          sig_fds[SIG_RD_FD] = sig_fds[SIG_WR_FD] = -1;
+     }
+
+
+     if(sig_fds[SIG_WR_FD] >= 0)
+     {
+          /* Re-direct SIGCHLD events through a pipe to be see by select() */
+          signal(SIGCHLD, sig_to_fd);
+     }
+
      if (repeats < 1) repeats = 1;
 
      while (repeats > 0)
@@ -1426,9 +1545,13 @@ int main(int argc, char **argv)
 
 	  if (flags & H_SWALLOW)
 	  {
+               initWindowOwnerMarker();
+
+#ifdef USE_MUTEX_LOCK
 	       /* If we are swallowing a victim window we need to guard
 		  against more than one instance of helper running in
-		  parallel, this is done using a global mutex semaphore. */
+		  parallel, this is done using a global mutex semaphore.
+                  Although its not successful in all cases! */
 	       initSwallowMutex();
 
 	       /* Belt & braces, make sure we give semaphore on error. */
@@ -1439,7 +1562,6 @@ int main(int argc, char **argv)
                 * This do-while loop checks for that case - this
                 * is better than putting a wait in the take semaphore
                 * rechecking loop fixes Mozdev bug 20088 */
-
                do
                {
 	           takeSwallowMutex(); 
@@ -1448,7 +1570,10 @@ int main(int argc, char **argv)
 	           save_initial_windows();
                }
                while(!stillHaveMutex());
-
+#else
+	       /* Has to be done before forking */
+	       save_initial_windows();
+#endif
                XSelectInput(display, parentDetails.window, 
                                                       SubstructureRedirectMask);
 	       XSelectInput(display, wattr.root, SubstructureNotifyMask);
@@ -1457,7 +1582,9 @@ int main(int argc, char **argv)
 
 	  if ((pid = fork()) == -1)
           {
+#ifdef USE_MUTEX_LOCK
                giveSwallowMutex();  
+#endif
 	       exit(EX_UNAVAILABLE);
           }
      
@@ -1469,6 +1596,7 @@ int main(int argc, char **argv)
 	  }
 	  else
 	  {
+
 	       D("Waiting for pid=%d\n", pid);
 	       handle_app(pid);
 	       D("Wait done (repeats=%d, loops=%d)\n", repeats, loops);
