@@ -1,6 +1,12 @@
 /*****************************************************************************
- * Author:  Peter Leese <peter@leese.net>
- *          Fredrik Hübinette <hubbe@hubbe.net>
+ *
+ * Original author: Fredrik Hübinette <hubbe@hubbe.net>
+ *
+ * Current Authors: Louis Bavoil <bavoil@cs.utah.edu>
+ *                  Peter Leese <hubbe@hubbe.net>
+ *
+ * This code is based on and is a branch of plugger written 
+ * by Fredrik Hübinette <hubbe@hubbe.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,8 +21,10 @@
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
+ *
  *****************************************************************************/
 
+#include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
 #include <stdio.h>
@@ -53,6 +61,10 @@ static int repeatsResetVal;
 
 static int pipe_fd;
 static int flags;
+static int sig_fds[2];
+
+#define SIG_RD_FD  0
+#define SIG_WR_FD  1
 
 static struct 
 {
@@ -131,9 +143,9 @@ static void redraw(void)
 
      if(old_buttonsize != buttonsize)
      {
-	  old_buttonsize=buttonsize;
-	  XFillRectangle(dpy, topLevel, gc_white,
-			 0,0,(unsigned)attr.width, (unsigned)attr.height);
+          old_buttonsize=buttonsize;
+          XFillRectangle(dpy, topLevel, gc_white,
+                         0,0,(unsigned)attr.width, (unsigned)attr.height);
      }
     
 
@@ -153,7 +165,7 @@ static void redraw(void)
           int i;
 
           XFillPolygon(dpy, topLevel, c, points, 6,
-		  Convex, CoordModeOrigin);
+                  Convex, CoordModeOrigin);
 
           points[2].y++;
           points[5].x++;
@@ -180,7 +192,7 @@ static void redraw(void)
           const unsigned h = points[4].y - points[0].y;
 
           XFillPolygon(dpy, topLevel, c, points, 6,
-		  Convex, CoordModeOrigin);
+                  Convex, CoordModeOrigin);
 
           /* erase left bit of previous not pressed button */
           XFillRectangle(dpy, topLevel, gc_white, x-d, y-d, d, h+d);
@@ -213,19 +225,28 @@ static void my_start(char * command)
 
          XDestroyWindow(dpy, topLevel);
          close(ConnectionNumber(dpy));
-         close(pipe_fd);
+
+          if(sig_fds[0] >= 0)
+          {
+               signal(SIGCHLD, SIG_DFL);
+               close(sig_fds[0]);
+          }
+          if(sig_fds[1] >= 0)
+          {
+               close(sig_fds[1]);
+          }
 
          cmd[0] = (char *) nextHelper;
 
          snprintf(buffer, sizeof(buffer), "%d,%d,%d,%lu,%d,%d,%d,%d",
-	          flags,
-	          repeats,
-	          pipe_fd,
-	          (unsigned long int) parentDetails.window,
-	          (int) parentDetails.x,
-	          (int) parentDetails.y,
-	          (int) parentDetails.width,
-	          (int) parentDetails.height);
+                  flags,
+                  repeats,
+                  pipe_fd,
+                  (unsigned long int) parentDetails.window,
+                  (int) parentDetails.x,
+                  (int) parentDetails.y,
+                  (int) parentDetails.width,
+                  (int) parentDetails.height);
 
 
          cmd[1] = buffer;
@@ -233,52 +254,61 @@ static void my_start(char * command)
          cmd[3] = 0;
 
          D("Switching to helper\n");
-	 execvp(nextHelper, cmd);
-	 exit(EX_UNAVAILABLE);
+         execvp(nextHelper, cmd);
+         exit(EX_UNAVAILABLE);
      }
 
      /* else linker is used to launch command */
      pid=fork();
      if(pid == -1)
      {
-	  return;
+          return;
      }
      if(pid == 0)
      {
-	  char *cmd[4];
+          char *cmd[4];
           /* Child process, so as usually need to cleanup any inherited
            * file descriptors */
           if(dpy)
           {
                close(ConnectionNumber(dpy));
           }
+          if(sig_fds[0] >= 0)
+          {
+               close(sig_fds[0]);
+          }
+          if(sig_fds[1] >= 0)
+          {
+               close(sig_fds[1]);
+          }
+          close(pipe_fd);
 
           /* Redirect stdout & stderr to /dev/null */
           if(flags & H_NOISY)
           {
-	        const int ofd = open("/dev/null", O_RDONLY);
-	        if(ofd)
+                const int ofd = open("/dev/null", O_RDONLY);
+                if(ofd)
                 {
-	             dup2(ofd, 1);
-	             dup2(ofd, 2);
-	             close(ofd);
+                     dup2(ofd, 1);
+                     dup2(ofd, 2);
+                     close(ofd);
                 }
           }
 
-	  setpgid(pid, 0);
+          setpgid(pid, 0);
 
-	  cmd[0]="/bin/sh";
-	  cmd[1]="-c";
-	  cmd[2]=command;
-	  cmd[3]=0;
-	  execvp(cmd[0], cmd);
-	  exit(EX_UNAVAILABLE);
+          cmd[0]="/bin/sh";
+          cmd[1]="-c";
+          cmd[2]=command;
+          cmd[3]=0;
+          execvp(cmd[0], cmd);
+          exit(EX_UNAVAILABLE);
      }
      else
      {
-	  if(!repeats) 
+          if(!repeats) 
               repeats = repeatsResetVal;
-	  if((repeats > 0) && (repeats != MAXINT)) 
+          if((repeats > 0) && (repeats != MAXINT)) 
               repeats--;
      }
 }
@@ -350,22 +380,87 @@ static void sigdie(int sig)
 
 /******************************************************************************/
 /**
- * @brief Callback filter for X events
+ * Convert signal to message in pipe - Callback function attached as a signal 
+ * handler. This sends a message over a pipe about the signal that has 
+ * just happened. This allows signals to be handled by select() 
  *
- * Defines the X events that we wish to see. Returns true for everything. 
+ * @param[in] sig The signal
  *
- * @param[in] dpy The display pointer
- * @param[in] ev  Pointer to the event data
- * @param[in] arg Any associated event data
- *
- * @return Always returns true
+ * @return nothing
  *
  *****************************************************************************/
-static Bool AllXEventsPredicate(Display *dpy, XEvent *ev, char *arg)
+static void sig_to_fd(int sig)
 {
-     return True;
+    D("Signal %i routed to pipe\n", sig);
+    if(write(sig_fds[SIG_WR_FD], &sig, sizeof(int)) != sizeof(int))
+    {
+        D("Signal write to pipe failed\n");
+    }
 }
 
+/******************************************************************************/
+/**
+ * Checks for X events and processes them. Returns when no more events left
+ * to process.
+ *
+ * @param[in] command - The command to execute if play pressed.
+ *
+ * @return nothing
+ *
+ *****************************************************************************/
+static void check_x_events(char * command)
+{
+     int numEvents = XPending(dpy);
+
+     while(numEvents > 0)
+     {
+          XEvent ev;
+
+          XNextEvent(dpy, &ev);
+
+          switch(ev.type)
+          {
+          case ButtonPress:
+               if(ev.xbutton.button == 1)
+               {
+                    buttonDown = 1;
+                    redraw();  /* do first gives quicker visual feedback */
+                    XSync(dpy, False);
+
+                    my_start(command); 
+               }
+               break;
+
+          case ButtonRelease:
+               if(buttonDown)
+               {   
+                    buttonDown = 0;
+                    redraw();
+               }     
+               break;
+
+          case Expose:
+               if(ev.xexpose.count)
+                    break;
+          case ResizeRequest:
+          case MapNotify:
+               redraw();
+               break;
+  
+          default:
+               D("Unknown event %d\n",ev.type);
+               break;
+          }
+
+          /* If this is the last of this batch, check that more havent
+           * been added in the meantime as a result of an action */
+          numEvents--;
+          if(numEvents == 0)
+          {
+               numEvents = XPending(dpy);
+          }
+     }
+}
 
 /******************************************************************************/
 /** 
@@ -435,19 +530,19 @@ static void check_pipe_fd_events(void)
      n = read(pipe_fd, ((char *)& wintmp),  sizeof(wintmp));
      if (n < 0)
      {
-	  if (errno == EINTR)
+          if (errno == EINTR)
                return;
-	  exit(EX_UNAVAILABLE);
+          exit(EX_UNAVAILABLE);
      }
   
      if (n == 0)
      {
-	  exit(EX_UNAVAILABLE);
+          exit(EX_UNAVAILABLE);
      }
   
      if (n != sizeof(wintmp))
      {
-	  return;
+          return;
      }
 
      parentDetails.window = (Window) wintmp.window;
@@ -524,14 +619,14 @@ int main(int argc, char **argv)
      }
 
      i = sscanf(argv[1],"%d,%d,%d,%lu,%d,%d,%d,%d",
-	    &flags,
-	    &repeatsResetVal,
-	    &pipe_fd,
-	    &temp,
-	    (int *)&parentDetails.x,
-	    (int *)&parentDetails.y,
-	    (unsigned int *)&parentDetails.width,
-	    (unsigned int *)&parentDetails.height);
+            &flags,
+            &repeatsResetVal,
+            &pipe_fd,
+            &temp,
+            (int *)&parentDetails.x,
+            (int *)&parentDetails.y,
+            (unsigned int *)&parentDetails.width,
+            (unsigned int *)&parentDetails.height);
 
      if(i < 8)
      {
@@ -559,14 +654,14 @@ int main(int argc, char **argv)
      /* The application will use the $repeat variable */
      if (flags & H_REPEATCOUNT)
      {   
-	  repeatsResetVal = 1;
+          repeatsResetVal = 1;
      }
 
      if(!(dpy = XOpenDisplay(getenv("DISPLAY"))))
      {
-	  D("%s: unable to open display %s\n",
-	         argv[0], XDisplayName(getenv("DISPLAY")));
-	  exitEarly();
+          D("%s: unable to open display %s\n",
+                 argv[0], XDisplayName(getenv("DISPLAY")));
+          exitEarly();
      }
 
      /* Adjust the width & height to compensate for the window border
@@ -595,17 +690,17 @@ int main(int argc, char **argv)
      attr.override_redirect=0;
 
      topLevel = XCreateWindow(dpy,
-			      parentDetails.window,
-			      x, y,
-			      width, height, 
+                              parentDetails.window,
+                              x, y,
+                              width, height, 
                               WINDOW_BORDER_WIDTH, 
                               CopyFromParent,
-			      InputOutput, (Visual *) CopyFromParent,
-			      (unsigned long)(CWBorderPixel|
-			       CWEventMask|
-			       CWOverrideRedirect|
-			       CWBackPixel),
-			      &attr);
+                              InputOutput, (Visual *) CopyFromParent,
+                              (unsigned long)(CWBorderPixel|
+                              CWEventMask|
+                               CWOverrideRedirect|
+                               CWBackPixel),
+                              &attr);
 
      classhint.res_name = "mozplugger-linker";
      classhint.res_class = "mozplugger-linker";
@@ -648,91 +743,75 @@ int main(int argc, char **argv)
      XSetErrorHandler(die);
 
      old_pid=pid;
- 
+
+     if(pipe(sig_fds) != 0)
+     {
+          D("Failed to create pipe to handle signals");
+          sig_fds[SIG_RD_FD] = sig_fds[SIG_WR_FD] = -1;
+     }
+
      signal(SIGHUP, sigdie);
      signal(SIGINT, sigdie);
      signal(SIGTERM, sigdie);
 
+     if(sig_fds[SIG_WR_FD] >= 0)
+     {
+          /* Re-direct SIGCHLD events through a pipe to be see by select() */
+          signal(SIGCHLD, sig_to_fd);
+     }
 
      while(1)
      {
-	  struct timeval tv;
-	  fd_set fds;
-	  XEvent ev;
+          fd_set fds;
           int maxFd;
 
-	  if(pid != old_pid)
-	  {
-	       redraw();
-	       old_pid=pid;
-	  }
-
-	  FD_ZERO(&fds);
-	  FD_SET(ConnectionNumber(dpy),&fds);
-	  FD_SET(pipe_fd,&fds);
-
-          maxFd = ConnectionNumber(dpy);
-          maxFd = MAX(maxFd, pipe_fd);
-
-	  tv.tv_sec = 0;
-	  tv.tv_usec = 1000000/5; /* Check if the subprocess died 5 times / sec */
-
-	  if( select(maxFd + 1, &fds, NULL, NULL, &tv) > 0)
+          if(pid != old_pid)
           {
-	       if (FD_ISSET(pipe_fd, &fds))
-               {    
-	            check_pipe_fd_events();
-               }
+               redraw();
+               old_pid=pid;
           }
 
-	  if(pid != -1)
-	  {
-	       int status;
-	       if(waitpid(pid, &status, WNOHANG) > 0)
-	       {
-		    if(pid > 0)
-		    {
-		         pid = -1;
-			 if(repeats > 0) my_start(command);
-		    }
-	       }
-	  }
+          check_x_events(command);
 
-	  while(XCheckIfEvent(dpy, &ev, AllXEventsPredicate, NULL))
-	  {
-	       switch(ev.type)
-	       {
-	       case ButtonPress:
-		    if(ev.xbutton.button == 1)
-		    {
-			 buttonDown = 1;
-                         redraw();  /* do first gives quicker visual feedback */
-                         XSync(dpy, False);
+          FD_ZERO(&fds);
+          FD_SET(ConnectionNumber(dpy),&fds);
+          FD_SET(pipe_fd,&fds);
 
-			 my_start(command); 
-		    }
-		    break;
-	 
-               case ButtonRelease:
-                    if(buttonDown)
-                    {   
-                        buttonDown = 0;
-                        redraw();
-                    }     
-                    break;
+          maxFd = MAX(ConnectionNumber(dpy), pipe_fd);
 
-	       case Expose:
-		    if(ev.xexpose.count) break;
-	       case ResizeRequest:
-	       case MapNotify:
-		    redraw();
-		    break;
-	  
-	       default:
-		    D("Unknown event %d\n",ev.type);
-                    break;
-	       }
-	  }
-     }
+          if(sig_fds[SIG_RD_FD] >= 0)
+          {
+               FD_SET(sig_fds[SIG_RD_FD], &fds);
+               maxFd = MAX(maxFd, sig_fds[SIG_RD_FD]);
+          }
+
+          D("SELECT IN maxFd = %i\n", maxFd);
+          if( select(maxFd + 1, &fds, NULL, NULL, 0) > 0)
+          {
+               if (FD_ISSET(pipe_fd, &fds))
+               {    
+                    check_pipe_fd_events();
+               }
+               if(FD_ISSET(sig_fds[SIG_RD_FD], &fds))
+               {
+                   int sig;
+                   read(sig_fds[SIG_RD_FD], &sig, sizeof(int));
+               }
+          }
+          D("SELECT OUT\n");
+
+          if(pid != -1)
+          {
+               int status;
+               if(waitpid(pid, &status, WNOHANG) > 0)
+               {
+                    if(pid > 0)
+                    {
+                         pid = -1;
+                         if(repeats > 0) my_start(command);
+                    }
+               }
+          }
+    }
 }
 
