@@ -1,7 +1,12 @@
 /*****************************************************************************
- * Authors: Louis Bavoil <bavoil@cs.utah.edu>
- *          Peter Leese <peter@leese.net>
- *          Fredrik Hbinette <hubbe@hubbe.net>
+ *
+ * Original author: Fredrik Hbinette <hubbe@hubbe.net>
+ *
+ * Current Authors: Louis Bavoil <bavoil@cs.utah.edu>
+ *                  Peter Leese <hubbe@hubbe.net>
+ *
+ * This code is based on and is a branch of plugger written 
+ * by Fredrik Hbinette <hubbe@hubbe.net>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -2224,22 +2229,6 @@ NPError NPP_Initialize(void)
      return NPERR_NO_ERROR;
 }
 
-/*****************************************************************************/
-/**
- * @brief NPP Get Java Class
- *
- * The browser calls this function once; when the plug-in is loaded. It returns
- * the Java Class associated with this plug-in. In this case there is None.
- *
- * @return Always returns Null
- *
- *****************************************************************************/
-jref NPP_GetJavaClass()
-{
-     D("GetJavaClass\n");
-     return NULL;
-}
-
 /*****************************************************************************/
 /**
  * @brief NPP Shutdown
