@@ -210,17 +210,6 @@ void NPN_ReloadPlugins(NPBool reloadPages)
     CallNPN_ReloadPluginsProc(gNetscapeFuncs.reloadplugins, reloadPages);
 }
 
-JRIEnv* NPN_GetJavaEnv()
-{
-    return CallNPN_GetJavaEnvProc(gNetscapeFuncs.getJavaEnv);
-}
-
-jref NPN_GetJavaPeer(NPP instance)
-{
-    return CallNPN_GetJavaPeerProc(gNetscapeFuncs.getJavaPeer,
-                       instance);
-}
-
 #if 0 /* Peter Leese - new stuff commented out for now */
 
 void
@@ -470,14 +459,11 @@ Private_Print(NPP instance, NPPrint* platformPrint)
     NPP_Print(instance, platformPrint);
 }
 
-JRIGlobalRef
+void *
 Private_GetJavaClass(void)
 {
-    jref clazz = NPP_GetJavaClass();
-    if (clazz) {
-    JRIEnv* env = NPN_GetJavaEnv();
-    return JRI_NewGlobalRef(env, clazz);
-    }
+    /* This is no longer supported in the newer version of mozilla */
+    PLUGINDEBUGSTR("GetJavaClass");
     return NULL;
 }
 
@@ -584,8 +570,8 @@ NP_Initialize(NPNetscapeFuncs* nsTable, NPPluginFuncs* pluginFuncs)
         gNetscapeFuncs.memflush      = nsTable->memflush;
         gNetscapeFuncs.reloadplugins = nsTable->reloadplugins;
 
-        gNetscapeFuncs.getJavaEnv    = nsTable->getJavaEnv;
-        gNetscapeFuncs.getJavaPeer   = nsTable->getJavaPeer;
+        gNetscapeFuncs.getJavaEnv    = NULL;   /* No longer supported */
+        gNetscapeFuncs.getJavaPeer   = NULL;   /* No longer supported */
 
         gNetscapeFuncs.posturlnotify = nsTable->posturlnotify;
         gNetscapeFuncs.getvalue      = nsTable->getvalue;
