@@ -45,9 +45,6 @@
 #ifndef _NPAPI_H_
 #define _NPAPI_H_
 
-#include "jri.h"		/* Java Runtime Interface */
-
-
 #if defined(XP_UNIX) 
 #	include <stdio.h>
 #	include <X11/Xlib.h>
@@ -506,8 +503,6 @@ int16   NP_LOADDS NPP_HandleEvent(NPP instance, void* event);
 void    NP_LOADDS NPP_URLNotify(NPP instance, const char* url,
                                 NPReason reason, void* notifyData);
 
-jref    NP_LOADDS NPP_GetJavaClass(void);
-
 NPError NP_LOADDS NPP_GetValue(void * instance, NPPVariable variable, void *value);
 NPError NP_LOADDS NPP_SetValue(NPP instance, NPNVariable variable, void *value);
 
@@ -539,9 +534,6 @@ void    NP_LOADDS NPN_MemFree(void* ptr);
 uint32  NP_LOADDS NPN_MemFlush(uint32 size);
 void    NP_LOADDS NPN_ReloadPlugins(NPBool reloadPages);
 
-JRIEnv* NP_LOADDS NPN_GetJavaEnv(void);
-jref    NP_LOADDS NPN_GetJavaPeer(NPP instance);
-
 NPError NP_LOADDS NPN_GetValue(NPP instance, NPNVariable variable, void *value);
 NPError NP_LOADDS NPN_SetValue(NPP instance, NPPVariable variable, void *value);
 void    NP_LOADDS NPN_InvalidateRect(NPP instance, NPRect *invalidRect);
