@@ -62,7 +62,7 @@
 #include "npruntime.h"
 #endif
 
-#include "jri.h"
+//#include "jri.h"
 
 
 /******************************************************************************************
@@ -296,19 +296,13 @@ typedef void (* NP_LOADDS NPN_ReloadPluginsUPP)(NPBool reloadPages);
 #define CallNPN_ReloadPluginsProc(FUNC, ARG1)		\
 		(*(FUNC))((ARG1))	
 
+/* Peter Leese - obsolete functions types */
 /* NPN_GetJavaEnv */
-typedef JRIEnv* (* NP_LOADDS NPN_GetJavaEnvUPP)(void);
-#define NewNPN_GetJavaEnvProc(FUNC)		\
-		((NPN_GetJavaEnvUPP) (FUNC))
-#define CallNPN_GetJavaEnvProc(FUNC)		\
-		(*(FUNC))()	
+typedef void* (* NP_LOADDS NPN_GetJavaEnvUPP)(void);
 
 /* NPN_GetJavaPeer */
-typedef jref (* NP_LOADDS NPN_GetJavaPeerUPP)(NPP instance);
-#define NewNPN_GetJavaPeerProc(FUNC)		\
-		((NPN_GetJavaPeerUPP) (FUNC))
-#define CallNPN_GetJavaPeerProc(FUNC, ARG1)		\
-		(*(FUNC))((ARG1))	
+typedef void * (* NP_LOADDS NPN_GetJavaPeerUPP)(NPP instance);
+
 
 #if 0 /* Peter Leese - new stuff commented out for now */
 
@@ -523,7 +517,7 @@ typedef struct _NPPluginFuncs {
     NPP_PrintUPP print;
     NPP_HandleEventUPP event;
     NPP_URLNotifyUPP urlnotify;
-    JRIGlobalRef javaClass;
+    void * javaClass;             /* JRIGlobalRef replaced with void * */
     NPP_GetValueUPP getvalue;
     NPP_SetValueUPP setvalue;
 } NPPluginFuncs;
